@@ -26,9 +26,14 @@ type UI struct {
 	blue      *color.Color
 	summarize bool
 	noPrefix  bool
+	top       int
 }
 
-var progressRunes = []rune(`⠇⠏⠋⠙⠹⠸⠼⠴⠦⠧`)
+var (
+	progressRunes      = []rune(`⠇⠏⠋⠙⠹⠸⠼⠴⠦⠧`)
+	progressRunesOld   = []rune(`-\\|/`)
+	progressRunesCount = len(progressRunes)
+)
 
 // CreateStdoutUI creates UI for stdout
 func CreateStdoutUI(
@@ -41,6 +46,7 @@ func CreateStdoutUI(
 	constGC bool,
 	useSIPrefix bool,
 	noPrefix bool,
+	top int,
 ) *UI {
 	ui := &UI{
 		UI: &common.UI{
@@ -55,6 +61,7 @@ func CreateStdoutUI(
 		output:    output,
 		summarize: summarize,
 		noPrefix:  noPrefix,
+		top:       top,
 	}
 
 	ui.red = color.New(color.FgRed).Add(color.Bold)
@@ -68,6 +75,11 @@ func CreateStdoutUI(
 	return ui
 }
 
+func (ui *UI) UseOldProgressRunes() {
+	progressRunes = progressRunesOld
+	progressRunesCount = len(progressRunes)
+}
+
 // StartUILoop stub
 func (ui *UI) StartUILoop() error {
 	return nil
@@ -134,15 +146,17 @@ func (ui *UI) ListDevices(getter device.DevicesInfoGetter) error {
 // AnalyzePath analyzes recursively disk usage in given path
 func (ui *UI) AnalyzePath(path string, _ fs.Item) error {
 	var (
-		dir  fs.Item
-		wait sync.WaitGroup
+		dir             fs.Item
+		wait            sync.WaitGroup
+		updateStatsDone chan struct{}
 	)
+	updateStatsDone = make(chan struct{}, 1)
 
 	if ui.ShowProgress {
 		wait.Add(1)
 		go func() {
 			defer wait.Done()
-			ui.updateProgress()
+			ui.updateProgress(updateStatsDone)
 		}()
 	}
 
@@ -151,19 +165,45 @@ func (ui *UI) AnalyzePath(path string, _ fs.Item) error {
 		defer wait.Done()
 		dir = ui.Analyzer.AnalyzeDir(path, ui.CreateIgnoreFunc(), ui.ConstGC)
 		dir.UpdateStats(make(fs.HardLinkedItems, 10))
+		updateStatsDone <- struct{}{}
 	}()
 
 	wait.Wait()
 
-	if ui.summarize {
+	switch {
+	case ui.top > 0:
+		ui.printTopFiles(dir)
+	case ui.summarize:
 		ui.printTotalItem(dir)
-	} else {
+	default:
 		ui.showDir(dir)
 	}
 
 	return nil
 }
 
+// ReadFromStorage reads analysis data from persistent key-value storage
+func (ui *UI) ReadFromStorage(storagePath, path string) error {
+	storage := analyze.NewStorage(storagePath, path)
+	closeFn := storage.Open()
+	defer closeFn()
+
+	dir, err := storage.GetDirForPath(path)
+	if err != nil {
+		return err
+	}
+
+	switch {
+	case ui.top > 0:
+		ui.printTopFiles(dir)
+	case ui.summarize:
+		ui.printTotalItem(dir)
+	default:
+		ui.showDir(dir)
+	}
+	return nil
+}
+
 func (ui *UI) showDir(dir fs.Item) {
 	sort.Sort(sort.Reverse(dir.GetFiles()))
 
@@ -172,6 +212,13 @@ func (ui *UI) showDir(dir fs.Item) {
 	}
 }
 
+func (ui *UI) printTopFiles(file fs.Item) {
+	collected := analyze.CollectTopFiles(file, ui.top)
+	for _, file := range collected {
+		ui.printItemPath(file)
+	}
+}
+
 func (ui *UI) printTotalItem(file fs.Item) {
 	var lineFormat string
 	if ui.UseColors {
@@ -215,7 +262,7 @@ func (ui *UI) printItem(file fs.Item) {
 			lineFormat,
 			string(file.GetFlag()),
 			ui.formatSize(size),
-			ui.blue.Sprintf("/"+file.GetName()))
+			ui.blue.Sprint("/"+file.GetName()))
 	} else {
 		fmt.Fprintf(ui.output,
 			lineFormat,
@@ -225,6 +272,27 @@ func (ui *UI) printItem(file fs.Item) {
 	}
 }
 
+func (ui *UI) printItemPath(file fs.Item) {
+	var lineFormat string
+	if ui.UseColors {
+		lineFormat = "%20s %s\n"
+	} else {
+		lineFormat = "%9s %s\n"
+	}
+
+	var size int64
+	if ui.ShowApparentSize {
+		size = file.GetSize()
+	} else {
+		size = file.GetUsage()
+	}
+
+	fmt.Fprintf(ui.output,
+		lineFormat,
+		ui.formatSize(size),
+		file.GetPath())
+}
+
 // ReadAnalysis reads analysis report from JSON file
 func (ui *UI) ReadAnalysis(input io.Reader) error {
 	var (
@@ -299,18 +367,18 @@ func (ui *UI) showReadingProgress(doneChan chan struct{}) {
 
 		time.Sleep(100 * time.Millisecond)
 		i++
-		i %= 10
+		i %= progressRunesCount
 	}
 }
 
-func (ui *UI) updateProgress() {
+func (ui *UI) updateProgress(updateStatsDone <-chan struct{}) {
 	emptyRow := "\r"
 	for j := 0; j < 100; j++ {
 		emptyRow += " "
 	}
 
 	progressChan := ui.Analyzer.GetProgressChan()
-	doneChan := ui.Analyzer.GetDone()
+	analysisDoneChan := ui.Analyzer.GetDone()
 
 	var progress common.CurrentProgress
 
@@ -320,9 +388,23 @@ func (ui *UI) updateProgress() {
 
 		select {
 		case progress = <-progressChan:
-		case <-doneChan:
-			fmt.Fprint(ui.output, "\r")
-			return
+		case <-analysisDoneChan:
+			for {
+				fmt.Fprint(ui.output, emptyRow)
+				fmt.Fprintf(ui.output, "\r %s ", string(progressRunes[i]))
+				fmt.Fprint(ui.output, "Calculating disk usage...")
+				time.Sleep(100 * time.Millisecond)
+				i++
+				i %= progressRunesCount
+
+				select {
+				case <-updateStatsDone:
+					fmt.Fprint(ui.output, emptyRow)
+					fmt.Fprint(ui.output, "\r")
+					return
+				default:
+				}
+			}
 		}
 
 		fmt.Fprintf(ui.output, "\r %s ", string(progressRunes[i]))
@@ -334,7 +416,7 @@ func (ui *UI) updateProgress() {
 
 		time.Sleep(100 * time.Millisecond)
 		i++
-		i %= 10
+		i %= progressRunesCount
 	}
 }
 
@@ -404,7 +486,7 @@ func maxLength(list []*device.Device, keyGetter func(*device.Device) string) int
 	return maxLen
 }
 
-func maxInt(x int, y int) int {
+func maxInt(x, y int) int {
 	if x > y {
 		return x
 	}
