@@ -9,21 +9,39 @@ import (
 	"github.com/rivo/tview"
 )
 
-func (ui *UI) formatFileRow(item fs.Item, maxUsage int64, maxSize int64, marked bool) string {
-	var part int
+const (
+	blackOnWhite = "[black:white:-]"
+	whiteOnBlack = "[white:black:-]"
 
-	if ui.ShowApparentSize {
-		part = int(float64(item.GetSize()) / float64(maxSize) * 100.0)
-	} else {
-		part = int(float64(item.GetUsage()) / float64(maxUsage) * 100.0)
+	defaultColor     = "[-::]"
+	defaultColorBold = "[::b]"
+)
+
+func (ui *UI) formatFileRow(item fs.Item, maxUsage, maxSize int64, marked, ignored bool) string {
+	part := 0
+	if !ignored {
+		if ui.ShowApparentSize {
+			if size := item.GetSize(); size > 0 {
+				part = int(float64(size) / float64(maxSize) * 100.0)
+			}
+		} else {
+			if usage := item.GetUsage(); usage > 0 {
+				part = int(float64(usage) / float64(maxUsage) * 100.0)
+			}
+		}
 	}
 
 	row := string(item.GetFlag())
 
-	if ui.UseColors && !marked {
-		row += "[#e67100::b]"
+	numberColor := fmt.Sprintf(
+		"[%s::b]",
+		ui.resultRow.NumberColor,
+	)
+
+	if ui.UseColors && !marked && !ignored {
+		row += numberColor
 	} else {
-		row += "[::b]"
+		row += defaultColorBold
 	}
 
 	if ui.ShowApparentSize {
@@ -39,22 +57,22 @@ func (ui *UI) formatFileRow(item fs.Item, maxUsage int64, maxSize int64, marked
 	}
 
 	if ui.showItemCount {
-		if ui.UseColors && !marked {
-			row += "[#e67100::b]"
+		if ui.UseColors && !marked && !ignored {
+			row += numberColor
 		} else {
-			row += "[::b]"
+			row += defaultColorBold
 		}
 		row += fmt.Sprintf("%11s ", ui.formatCount(item.GetItemCount()))
 	}
 
 	if ui.showMtime {
-		if ui.UseColors && !marked {
-			row += "[#e67100::b]"
+		if ui.UseColors && !marked && !ignored {
+			row += numberColor
 		} else {
-			row += "[::b]"
+			row += defaultColorBold
 		}
 		row += fmt.Sprintf(
-			"%s [-::]",
+			"%s "+defaultColor,
 			item.GetMtime().Format("2006-01-02 15:04:05"),
 		)
 	}
@@ -69,29 +87,33 @@ func (ui *UI) formatFileRow(item fs.Item, maxUsage int64, maxSize int64, marked
 	}
 
 	if item.IsDir() {
-		if ui.UseColors && !marked {
-			row += "[#3498db::b]/"
+		if ui.UseColors && !marked && !ignored {
+			row += fmt.Sprintf("[%s::b]/", ui.resultRow.DirectoryColor)
 		} else {
-			row += "[::b]/"
+			row += defaultColorBold + "/"
 		}
 	}
 	row += tview.Escape(item.GetName())
 	return row
 }
 
-func (ui *UI) formatSize(size int64, reverseColor bool, transparentBg bool) string {
+func (ui *UI) formatSize(size int64, reverseColor, transparentBg bool) string {
 	var color string
 	if reverseColor {
 		if ui.UseColors {
-			color = "[black:#2479d0:-]"
+			color = fmt.Sprintf(
+				"[%s:%s:-]",
+				ui.footerTextColor,
+				ui.footerBackgroundColor,
+			)
 		} else {
-			color = "[black:white:-]"
+			color = blackOnWhite
 		}
 	} else {
 		if transparentBg {
-			color = "[-::]"
+			color = defaultColor
 		} else {
-			color = "[white:black:-]"
+			color = whiteOnBlack
 		}
 	}
 
@@ -103,7 +125,7 @@ func (ui *UI) formatSize(size int64, reverseColor bool, transparentBg bool) stri
 
 func (ui *UI) formatCount(count int) string {
 	row := ""
-	color := "[-::]"
+	color := defaultColor
 	count64 := float64(count)
 
 	switch {
