@@ -13,7 +13,7 @@ func (ui *UI) keyPressed(key *tcell.EventKey) *tcell.EventKey {
 		return nil
 	}
 
-	if ui.pages.HasPage("file") {
+	if ui.pages.HasPage("file") || ui.pages.HasPage("export") {
 		return key // send event to primitive
 	}
 	if ui.filtering {
@@ -33,8 +33,11 @@ func (ui *UI) keyPressed(key *tcell.EventKey) *tcell.EventKey {
 		return nil
 	}
 
-	if ui.pages.HasPage("confirm") ||
-		ui.pages.HasPage("progress") ||
+	if ui.pages.HasPage("confirm") {
+		return ui.handleConfirmation(key)
+	}
+
+	if ui.pages.HasPage("progress") ||
 		ui.pages.HasPage("deleting") ||
 		ui.pages.HasPage("emptying") {
 		return key
@@ -83,6 +86,16 @@ func (ui *UI) handleClosingModals(key *tcell.EventKey) *tcell.EventKey {
 	return key
 }
 
+func (ui *UI) handleConfirmation(key *tcell.EventKey) *tcell.EventKey {
+	if key.Rune() == 'h' {
+		return tcell.NewEventKey(tcell.KeyLeft, 0, 0)
+	}
+	if key.Rune() == 'l' {
+		return tcell.NewEventKey(tcell.KeyRight, 0, 0)
+	}
+	return key
+}
+
 func (ui *UI) handleInfoPageEvents(key *tcell.EventKey) *tcell.EventKey {
 	if ui.pages.HasPage("info") {
 		switch key.Rune() {
@@ -187,6 +200,7 @@ func (ui *UI) handleFiltering(key *tcell.EventKey) *tcell.EventKey {
 	return key
 }
 
+// nolint: funlen // Why: there's a lot of options to handle
 func (ui *UI) handleMainActions(key *tcell.EventKey) *tcell.EventKey {
 	switch key.Rune() {
 	case 'd':
@@ -231,6 +245,9 @@ func (ui *UI) handleMainActions(key *tcell.EventKey) *tcell.EventKey {
 		if ui.currentDir != nil {
 			ui.rescanDir()
 		}
+	case 'E':
+		ui.confirmExport()
+		return nil
 	case 's':
 		ui.setSorting("size")
 	case 'C':
@@ -244,6 +261,8 @@ func (ui *UI) handleMainActions(key *tcell.EventKey) *tcell.EventKey {
 		return nil
 	case ' ':
 		ui.handleMark()
+	case 'I':
+		ui.ignoreItem()
 	}
 	return key
 }
@@ -308,3 +327,24 @@ func (ui *UI) handleMark() {
 
 	ui.fileItemMarked(row)
 }
+
+func (ui *UI) ignoreItem() {
+	if ui.currentDir == nil {
+		return
+	}
+	// do not allow ignoring parent dir
+	row, column := ui.table.GetSelection()
+	selectedFile, ok := ui.table.GetCell(row, column).GetReference().(fs.Item)
+	if !ok || selectedFile == ui.currentDir.GetParent() {
+		return
+	}
+
+	if _, ok := ui.ignoredRows[row]; ok {
+		delete(ui.ignoredRows, row)
+	} else {
+		ui.ignoredRows[row] = struct{}{}
+	}
+	ui.showDir()
+	// select next row if possible
+	ui.table.Select(min(row+1, ui.table.GetRowCount()-1), 0)
+}
