@@ -2,6 +2,12 @@ package tui
 
 import (
 	"io"
+	"os"
+	"os/signal"
+	"runtime"
+	"sync"
+	"syscall"
+	"time"
 
 	"golang.org/x/exp/slices"
 
@@ -11,6 +17,7 @@ import (
 	"github.com/dundee/gdu/v5/pkg/analyze"
 	"github.com/dundee/gdu/v5/pkg/device"
 	"github.com/dundee/gdu/v5/pkg/fs"
+	"github.com/dundee/gdu/v5/pkg/remove"
 	"github.com/gdamore/tcell/v2"
 	"github.com/rivo/tview"
 )
@@ -21,12 +28,14 @@ type UI struct {
 	app                     common.TermApplication
 	screen                  tcell.Screen
 	output                  io.Writer
+	grid                    *tview.Grid
 	header                  *tview.TextView
 	footer                  *tview.Flex
 	footerLabel             *tview.TextView
 	currentDirLabel         *tview.TextView
 	pages                   *tview.Pages
 	progress                *tview.TextView
+	status                  *tview.TextView
 	help                    *tview.Flex
 	table                   *tview.Table
 	filteringInput          *tview.InputField
@@ -51,12 +60,38 @@ type UI struct {
 	linkedItems             fs.HardLinkedItems
 	selectedTextColor       tcell.Color
 	selectedBackgroundColor tcell.Color
+	footerTextColor         string
+	footerBackgroundColor   string
+	footerNumberColor       string
+	headerTextColor         string
+	headerBackgroundColor   string
+	headerHidden            bool
+	resultRow               ResultRow
 	currentItemNameMaxLen   int
 	useOldSizeBar           bool
 	defaultSortBy           string
 	defaultSortOrder        string
+	ignoredRows             map[int]struct{}
 	markedRows              map[int]struct{}
 	exportName              string
+	noDelete                bool
+	deleteInBackground      bool
+	deleteQueue             chan deleteQueueItem
+	activeWorkers           int
+	workersMut              sync.Mutex
+	statusMut               sync.RWMutex
+	deleteWorkersCount      int
+}
+
+type deleteQueueItem struct {
+	item        fs.Item
+	shouldEmpty bool
+}
+
+// ResultRow is a struct for a row in the result table
+type ResultRow struct {
+	NumberColor    string
+	DirectoryColor string
 }
 
 // Option is optional function customizing the bahaviour of UI
@@ -88,8 +123,8 @@ func CreateUI(
 		output:                  output,
 		askBeforeDelete:         true,
 		showItemCount:           false,
-		remover:                 analyze.RemoveItemFromDir,
-		emptier:                 analyze.EmptyFileFromDir,
+		remover:                 remove.ItemFromDir,
+		emptier:                 remove.EmptyFileFromDir,
 		exec:                    Execute,
 		linkedItems:             make(fs.HardLinkedItems, 10),
 		selectedTextColor:       tview.Styles.TitleColor,
@@ -97,8 +132,12 @@ func CreateUI(
 		currentItemNameMaxLen:   70,
 		defaultSortBy:           "size",
 		defaultSortOrder:        "desc",
+		ignoredRows:             make(map[int]struct{}),
 		markedRows:              make(map[int]struct{}),
 		exportName:              "export.json",
+		noDelete:                false,
+		deleteQueue:             make(chan deleteQueueItem, 1000),
+		deleteWorkersCount:      3 * runtime.GOMAXPROCS(0),
 	}
 	for _, o := range opts {
 		o(ui)
@@ -114,19 +153,10 @@ func CreateUI(
 	ui.app.SetInputCapture(ui.keyPressed)
 	ui.app.SetMouseCapture(ui.onMouse)
 
-	var textColor, textBgColor tcell.Color
-	if ui.UseColors {
-		textColor = tcell.NewRGBColor(0, 0, 0)
-		textBgColor = tcell.NewRGBColor(36, 121, 208)
-	} else {
-		textColor = tcell.NewRGBColor(0, 0, 0)
-		textBgColor = tcell.NewRGBColor(255, 255, 255)
-	}
-
 	ui.header = tview.NewTextView()
 	ui.header.SetText(" gdu ~ Use arrow keys to navigate, press ? for help ")
-	ui.header.SetTextColor(textColor)
-	ui.header.SetBackgroundColor(textBgColor)
+	ui.header.SetTextColor(tcell.GetColor(ui.headerTextColor))
+	ui.header.SetBackgroundColor(tcell.GetColor(ui.headerBackgroundColor))
 
 	ui.currentDirLabel = tview.NewTextView()
 	ui.currentDirLabel.SetTextColor(tcell.ColorDefault)
@@ -137,7 +167,6 @@ func CreateUI(
 	ui.table.SetSelectedFunc(ui.fileItemSelected)
 
 	if ui.UseColors {
-
 		ui.table.SetSelectedStyle(tcell.Style{}.
 			Foreground(ui.selectedTextColor).
 			Background(ui.selectedBackgroundColor).Bold(true))
@@ -148,21 +177,17 @@ func CreateUI(
 	}
 
 	ui.footerLabel = tview.NewTextView().SetDynamicColors(true)
-	ui.footerLabel.SetTextColor(textColor)
-	ui.footerLabel.SetBackgroundColor(textBgColor)
+	ui.footerLabel.SetTextColor(tcell.GetColor(ui.footerTextColor))
+	ui.footerLabel.SetBackgroundColor(tcell.GetColor(ui.footerBackgroundColor))
 	ui.footerLabel.SetText(" No items to display. ")
 
 	ui.footer = tview.NewFlex()
 	ui.footer.AddItem(ui.footerLabel, 0, 1, false)
 
-	grid := tview.NewGrid().SetRows(1, 1, 0, 1).SetColumns(0)
-	grid.AddItem(ui.header, 0, 0, 1, 1, 0, 0, false).
-		AddItem(ui.currentDirLabel, 1, 0, 1, 1, 0, 0, false).
-		AddItem(ui.table, 2, 0, 1, 1, 0, 0, true).
-		AddItem(ui.footer, 3, 0, 1, 1, 0, 0, false)
+	ui.createGrid()
 
 	ui.pages = tview.NewPages().
-		AddPage("background", grid, true, true)
+		AddPage("background", ui.grid, true, true)
 	ui.pages.SetBackgroundColor(tcell.ColorDefault)
 
 	ui.app.SetRoot(ui.pages, true)
@@ -170,6 +195,22 @@ func CreateUI(
 	return ui
 }
 
+// createGrid creates the main grid layout
+func (ui *UI) createGrid() {
+	if ui.headerHidden {
+		ui.grid = tview.NewGrid().SetRows(1, 0, 1).SetColumns(0)
+		ui.grid.AddItem(ui.currentDirLabel, 0, 0, 1, 1, 0, 0, false).
+			AddItem(ui.table, 1, 0, 1, 1, 0, 0, true).
+			AddItem(ui.footer, 2, 0, 1, 1, 0, 0, false)
+	} else {
+		ui.grid = tview.NewGrid().SetRows(1, 1, 0, 1).SetColumns(0)
+		ui.grid.AddItem(ui.header, 0, 0, 1, 1, 0, 0, false).
+			AddItem(ui.currentDirLabel, 1, 0, 1, 1, 0, 0, false).
+			AddItem(ui.table, 2, 0, 1, 1, 0, 0, true).
+			AddItem(ui.footer, 3, 0, 1, 1, 0, 0, false)
+	}
+}
+
 // SetSelectedTextColor sets the color for the highighted selected text
 func (ui *UI) SetSelectedTextColor(color tcell.Color) {
 	ui.selectedTextColor = color
@@ -180,10 +221,50 @@ func (ui *UI) SetSelectedBackgroundColor(color tcell.Color) {
 	ui.selectedBackgroundColor = color
 }
 
+// SetFooterTextColor sets the color for the footer text
+func (ui *UI) SetFooterTextColor(color string) {
+	ui.footerTextColor = color
+}
+
+// SetFooterBackgroundColor sets the color for the footer background
+func (ui *UI) SetFooterBackgroundColor(color string) {
+	ui.footerBackgroundColor = color
+}
+
+// SetFooterNumberColor sets the color for the footer number
+func (ui *UI) SetFooterNumberColor(color string) {
+	ui.footerNumberColor = color
+}
+
+// SetHeaderTextColor sets the color for the header text
+func (ui *UI) SetHeaderTextColor(color string) {
+	ui.headerTextColor = color
+}
+
+// SetHeaderBackgroundColor sets the color for the header background
+func (ui *UI) SetHeaderBackgroundColor(color string) {
+	ui.headerBackgroundColor = color
+}
+
+// SetHeaderHidden sets the flag to hide the header
+func (ui *UI) SetHeaderHidden() {
+	ui.headerHidden = true
+}
+
+// SetResultRowDirectoryColor sets the color for the result row directory
+func (ui *UI) SetResultRowDirectoryColor(color string) {
+	ui.resultRow.DirectoryColor = color
+}
+
+// SetResultRowNumberColor sets the color for the result row number
+func (ui *UI) SetResultRowNumberColor(color string) {
+	ui.resultRow.NumberColor = color
+}
+
 // SetCurrentItemNameMaxLen sets the maximum length of the path of the currently processed item
 // to be shown in the progress modal
-func (ui *UI) SetCurrentItemNameMaxLen(len int) {
-	ui.currentItemNameMaxLen = len
+func (ui *UI) SetCurrentItemNameMaxLen(maxLen int) {
+	ui.currentItemNameMaxLen = maxLen
 }
 
 // UseOldSizeBar uses the old size bar (# chars) instead of the new one (unicode block elements)
@@ -197,11 +278,61 @@ func (ui *UI) SetChangeCwdFn(fn func(string) error) {
 	ui.changeCwdFn = fn
 }
 
+// SetDeleteInParallel sets the flag to delete files in parallel
+func (ui *UI) SetDeleteInParallel() {
+	ui.remover = remove.ItemFromDirParallel
+}
+
 // StartUILoop starts tview application
 func (ui *UI) StartUILoop() error {
+	go func() {
+		c := make(chan os.Signal, 1)
+		signal.Notify(
+			c,
+			syscall.SIGHUP,
+			syscall.SIGINT,
+			syscall.SIGQUIT,
+			syscall.SIGILL,
+			syscall.SIGTRAP,
+			syscall.SIGABRT,
+			syscall.SIGPIPE,
+			syscall.SIGTERM,
+		)
+		s := <-c
+		log.Printf("Got signal: %s", s)
+		ui.app.QueueUpdateDraw(func() {
+			ui.app.Stop()
+		})
+	}()
+
 	return ui.app.Run()
 }
 
+// SetShowItemCount sets the flag to show number of items in directory
+func (ui *UI) SetShowItemCount() {
+	ui.showItemCount = true
+}
+
+// SetShowMTime sets the flag to show last modification time of items in directory
+func (ui *UI) SetShowMTime() {
+	ui.showMtime = true
+}
+
+// SetNoDelete disables all write operations
+func (ui *UI) SetNoDelete() {
+	ui.noDelete = true
+}
+
+// SetDeleteInBackground sets the flag to delete files in background
+func (ui *UI) SetDeleteInBackground() {
+	ui.deleteInBackground = true
+
+	for i := 0; i < ui.deleteWorkersCount; i++ {
+		go ui.deleteWorker()
+	}
+	go ui.updateStatusWorker()
+}
+
 func (ui *UI) resetSorting() {
 	ui.sortBy = ui.defaultSortBy
 	ui.sortOrder = ui.defaultSortOrder
@@ -237,6 +368,7 @@ func (ui *UI) fileItemSelected(row, column int) {
 	ui.currentDir = selectedDir
 	ui.hideFilterInput()
 	ui.markedRows = make(map[int]struct{})
+	ui.ignoredRows = make(map[int]struct{})
 	ui.showDir()
 
 	if origDir.GetParent() != nil && selectedDir.GetName() == origDir.GetParent().GetName() {
@@ -262,7 +394,6 @@ func (ui *UI) deviceItemSelected(row, column int) {
 
 	paths := device.GetNestedMountpointsPaths(selectedDevice.MountPoint, ui.devices)
 	ui.IgnoreDirPathPatterns, err = common.CreateIgnorePattern(paths)
-
 	if err != nil {
 		log.Printf("Creating path patterns for other devices failed: %s", paths)
 	}
@@ -278,6 +409,23 @@ func (ui *UI) deviceItemSelected(row, column int) {
 }
 
 func (ui *UI) confirmDeletion(shouldEmpty bool) {
+	if ui.noDelete {
+		previousHeaderText := ui.header.GetText(false)
+
+		// show feedback to user
+		ui.header.SetText(" Deletion is disabled!")
+
+		go func() {
+			time.Sleep(2 * time.Second)
+			ui.app.QueueUpdateDraw(func() {
+				ui.header.Clear()
+				ui.header.SetText(previousHeaderText)
+			})
+		}()
+
+		return
+	}
+
 	if len(ui.markedRows) > 0 {
 		ui.confirmDeletionMarked(shouldEmpty)
 	} else {
